clear all
tic

dim = 4;
nobs = 2000;
nMC = 100;
startMC = 1;
df = 6;
max_iter = 50;

% Preallocate memory
Y_mat = zeros(nobs,dim);
resid_mat = zeros(nobs,dim);
beta_OLS = zeros(dim*3,nMC);
beta_GLS = zeros(dim*3,nMC);
sigma_OLS = zeros(dim*(dim+1)/2,nMC);
sigma_GLS = zeros(dim*(dim+1)/2,nMC);


sigma_test = kron( ones(dim,dim),sparse(eye(nobs)) );
[row_index,col_index] = find(sigma_test);
clear sigma_test

% Simulation begins
H_status_bar = waitbar(0,'Simulation  ');
for r = startMC:nMC
    
    waitbar(r/nMC,H_status_bar,['Simulation   ',num2str(r)]);
    
    % Generating data
    rand('seed',r*12345);
    randn('seed',r*1234567);
    sigma = 0.3*ones(dim,dim) + 0.7*eye(dim);
    epsilon = mvtrnd(sigma,df,nobs);
    X_SUR = [];
    for m = 1:dim
        X = [ones(nobs,1),randn(nobs,1),randn(nobs,1)];        
        Y = X * [m;m;m] + epsilon(:,m);
        Y(Y<0) = 0;
        Y_mat(:,m) = Y;
        X_SUR = blkdiag(X_SUR,X);
    end    
    Y_SUR = Y_mat(:);
    
    % OLS estimation
    beta_OLS(:,r) = (X_SUR' * X_SUR) \ (X_SUR' * Y_SUR);
    resid = Y_SUR - X_SUR * beta_OLS(:,r);
    resid_mat(:) = resid;
    sigma_OLS_mat = resid_mat' * resid_mat / nobs;
    sigma_OLS(:,r) = vech(sigma_OLS_mat);
    
    % GLS estimation
    inv_sigma = inv(sigma_OLS_mat);
    beta_update = beta_OLS(:,r);
    for m = 1:max_iter
        
        beta_temp = beta_update;
        
        rep_inv_sigma = repmat(inv_sigma,nobs,1);        
        sigma_matrix_sparse = sparse(row_index,col_index,rep_inv_sigma(:),dim*nobs,dim*nobs);
        beta_update = (X_SUR' * sigma_matrix_sparse * X_SUR) \ (X_SUR' * sigma_matrix_sparse * Y_SUR);
        
        resid = Y_SUR - X_SUR * beta_update;
        resid_mat(:) = resid;
        sigma_update = resid_mat' * resid_mat / nobs;
        inv_sigma = inv(sigma_update);
        
        
        if norm(beta_update - beta_temp) < 1e-8
            disp(['After ',num2str(m),' iterations, converged ! '])
            beta_GLS(:,r) = beta_update;
            sigma_GLS(:,r) = vech(sigma_update);
            break
        end
    end
        
end
close(H_status_bar);

% calculate statistics
para_true = [reshape( repmat(1:dim,3,1), dim*3,1);nan;vech(sigma)];
para_OLS = [beta_OLS;nan(1,nMC);sigma_OLS];
para_GLS = [beta_GLS;nan(1,nMC);sigma_GLS];
para_OLS_mean = mean(para_OLS,2);
para_GLS_mean = mean(para_GLS,2);
para_OLS_std = std(para_OLS,0,2);
para_GLS_std = std(para_GLS,0,2);
para_OLS_MSE =  sum( (para_OLS - para_true(:,ones(nMC,1))).^2 ,2) / nMC;
para_GLS_MSE =  sum( (para_GLS - para_true(:,ones(nMC,1))).^2 ,2) / nMC;

stat_pool = [para_OLS_mean,para_OLS_std,para_OLS_MSE,...
    para_GLS_mean,para_GLS_std,para_GLS_MSE];

% Page 1: summary statistics
% heading
npara = dim*3 + dim*(dim+1)/2;
row_heading = {'OLS','OLS_STD','OLS_MSE','GLS','GLS_STD','GLS_MSE'};
col_heading = cell(npara+1,1);
k = 1;
for m = 1:dim
    for n = 1:3
        col_heading{k} = ['',num2str(m),num2str(n)];
        k = k + 1;
    end
end
k = k + 1;
for m = 1:dim
    for n = m:dim
        col_heading{k} = ['',num2str(n),num2str(m)];
        k = k + 1;
    end
end

xlswrite('Sim OLS GLS.xls',row_heading,'Summary','B1')
xlswrite('Sim OLS GLS.xls',col_heading,'Summary','A2')
xlswrite('Sim OLS GLS.xls',stat_pool,'Summary','B2')


% Page 2-4: simulation details
xlswrite('Sim OLS GLS.xls',startMC:nMC,'OLS',[native2unicode(65+startMC),'1'])
xlswrite('Sim OLS GLS.xls',startMC:nMC,'GLS',[native2unicode(65+startMC),'1'])
xlswrite('Sim OLS GLS.xls',col_heading,'OLS','A2')
xlswrite('Sim OLS GLS.xls',col_heading,'GLS','A2')
xlswrite('Sim OLS GLS.xls',para_OLS(:,startMC:nMC),'OLS',[native2unicode(65+startMC),'2'])
xlswrite('Sim OLS GLS.xls',para_GLS(:,startMC:nMC),'GLS',[native2unicode(65+startMC),'2'])

